/*
 * bufpointpainter.cc
 * 
 * Point caching XPainter for performant point drawing (single color). 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "bufpointpainter.h"

#include <stdlib.h>
#include <assert.h>

#include "qtxalloc.h"

namespace QTX
{

void BufferedPointPainter::Flush()
{
	if(np>1)
	{  DrawPoints(p,np);  }
	else if(np)
	{  DrawPoint(p->x,p->y);  }
	np=0;
}


void BufferedPointPainter::Setup(uint cache_size)
{
	if(np)
	{  Flush();  }
	Free(p);
	csize=cache_size ? cache_size : 1;
	p=(XPoint*)Malloc(csize*sizeof(XPoint));
}


BufferedPointPainter::BufferedPointPainter(const QPaintDevice *pd,
	uint cache_size) : 
	XPainter(pd)
{
	csize=cache_size ? cache_size : 1;
	p=(XPoint*)Malloc(csize*sizeof(XPoint));
	np=0;
}

BufferedPointPainter::~BufferedPointPainter()
{
	Free(p);
	p=NULL;
}

}  // end of namespace QTX
