/*
 * bufpointpainter.h
 * 
 * Point caching XPainter for performant point drawing (single color). 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _INC_QTXLIB_BufferedPointPainter_H_
#define _INC_QTXLIB_BufferedPaintPainter_H_ 1

#include "xpainter.h"

namespace QTX
{

// Class which allows you to draw points via XDrawPoints using a 
// point cache. 
// There is exactly one XPainter per BufferedPointPainter. 
// The cache is automatically flushed if you draw more than it can 
// hold; you may also flush it explicitly. 
class BufferedPointPainter : public XPainter
{
	private:
		uint csize;    // size of cache 
		uint np;       // number of used points
		XPoint *p;
	public:
		// See Setup() for explanation on cache_size & fill args. 
		BufferedPointPainter(const QPaintDevice *pd,uint cache_size);
		~BufferedPointPainter();
		
		uint GetCacheSize() const
			{  return(csize);  }
		
		// cache_size: number of XPoints in cache. 
		// This allocates cache_size*sizeof(XPoint) bytes. 
		// You cannot use cache_size=0. 
		void Setup(uint cache_size);
		
		// Flush = draw the points: 
		void Flush();
		
		// Discart cache content without flushing: 
		void Clear()
			{  np=0;  }
		
		// Add a point to the cache: 
		void AddPoint(int x,int y)
		{
			if(np>=csize)  Flush();
			XPoint *cp=&p[np++];
			cp->x=x;  cp->y=y;
		}
};

}  // end of namespace QTX

#endif   /* _INC_QTXLIB_BufferedPointPainter_H_ */
