/*
 * bufrectpainter.cc
 * 
 * Rectangle caching XPainter for performant rectangle drawing 
 * (single color). 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "bufrectpainter.h"

#include <stdlib.h>
#include <assert.h>

#include "qtxalloc.h"


namespace QTX
{

void BufferedRectPainter::Flush()
{
	if(fill)
	{
		if(nr>1)
		{  FillRectangles(r,nr);  }
		else if(nr)
		{  FillRectangle(r->x,r->y,r->width,r->height);  }
	}
	else
	{
		if(nr>1)
		{  DrawRectangles(r,nr);  }
		else if(nr)
		{  DrawRectangle(r->x,r->y,r->width,r->height);  }
	}
	nr=0;
}


void BufferedRectPainter::Setup(uint cache_size,bool _fill)
{
	if(nr)
	{  Flush();  }
	Free(r);
	csize=cache_size ? cache_size : 1;
	r=(XRectangle*)Malloc(csize*sizeof(XRectangle));
	fill=_fill;
}


BufferedRectPainter::BufferedRectPainter(const QPaintDevice *pd,
	uint cache_size,bool _fill) : 
	XPainter(pd)
{
	csize=cache_size ? cache_size : 1;
	fill=_fill;
	r=(XRectangle*)Malloc(csize*sizeof(XRectangle));
	nr=0;
}

BufferedRectPainter::~BufferedRectPainter()
{
	Free(r);
	r=NULL;
}

}  // end of namespace QTX
