/*
 * rgbcolorsource.cc
 * 
 * Converting RGB values to pixel values for visuals which do not need 
 * color allocation. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "rgbcolorsource.h"

#include <stdio.h>
#include <stdlib.h>


namespace QTX
{

void RGBColorSource::_setup_cmask(ulong rm,ulong gm,ulong bm)
{
	if(!rm || !gm || !bm)
	{
		fprintf(stderr,"Illegal color masks: %lx,%lx,%lx\n",rm,gm,bm);
		abort();
	}
	
	rmask=rm;   gmask=gm;     bmask=bm;
	nred=0LU;   ngreen=0LU;   nblue=0LU;
	rshift=-1;  gshift=-1;    bshift=-1;
	
	int nb=sizeof(ulong)*8;
	for(int i=0; i<nb; i++)
	{
		register ulong tst=1LU<<i;
		
		nred+=   (rmask & tst) >> i;
		ngreen+= (gmask & tst) >> i;
		nblue+=  (bmask & tst) >> i;
		
		if( rshift<0 && (rmask & tst) )  {  rshift=i;  }
		if( gshift<0 && (gmask & tst) )  {  gshift=i;  }
		if( bshift<0 && (bmask & tst) )  {  bshift=i;  }
	}
	
    rss = (16 - nred);
    gss = (16 - ngreen);
    bss = (16 - nblue);
    
	if(nred)    {  nred   = 1LU << nred;    }
	if(ngreen)  {  ngreen = 1LU << ngreen;  }
	if(nblue)   {  nblue  = 1LU << nblue;   }
	
	nred_1   = nred   - 1LU;
	ngreen_1 = ngreen - 1LU;
	nblue_1  = nblue  - 1LU;
}


void RGBColorSource::_setup(Visual *vis)
{
	switch(vis->c_class)
	{
        case TrueColor:
        case DirectColor:
			// Okay, these are good visuals: No need for color 
			// allocation and the stuff. (AFAIK, at least)
			break;
		case StaticGray:
        case StaticColor:
			// I don't know about these, to setting drect flag to 
			// false if the secure way: 
			fprintf(stderr,"Not using direct rgb color - can we? %lx,%lx,%lx\n",
				vis->red_mask,vis->green_mask,vis->blue_mask);
			break;
        case PseudoColor:
        case GrayScale:
        default:
			fprintf(stderr,"Using not supported visual %d (%lx,%lx,%lx)\n",
				vis->c_class,
				vis->red_mask,vis->green_mask,vis->blue_mask);
			abort();
			break;
	}
	
	_setup_cmask(vis->red_mask,vis->green_mask,vis->blue_mask);
}

}  // end of namespace QTX
