/*
 * corebench.cc
 * 
 * Simple burn core benchmark program for code optimization issues. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include <stdio.h>
#include <assert.h>
#include <sys/time.h>

#include "burncore.h"

int main(int argc,char **arg)
{
	double tprop=0.5;
	int width=300,height=300;
	if(argc>=2)  tprop=atof(arg[1]);
	if(argc>=3)  width=atoi(arg[2]);
	if(argc>=4)  height=atoi(arg[3]);
	
	if(sizeof(BCell)!=8)
	{  fprintf(stderr,"Warning: sizeof(BCell)=%u\n",sizeof(BCell));  }
	
	fprintf(stderr,"Setup....");
	
	BurnArea area;
	srandom(0xcafebabe);
	area.SetSize(width,height,0);
	area.RandomFill(tprop,/*mass=*/10);
	
	fprintf(stderr,"OK\nArea %dx%d, tree prop %f\n",
		area.AreaWidth(),area.AreaHeight(),tprop);
	
	area.SetFire(width/2,height/2);
	area.SetFire(width/2+1,height/2);
	area.SetFire(width/2,height/2+1);
	area.SetFire(width/2+1,height/2+1);
	
	{
		timeval start,stop;
		gettimeofday(&start,NULL);
		int niter=0;
		for(;;)
		{
			int rv=area.SimTick();
			++niter;
			if(!rv)  break;
		}
		gettimeofday(&stop,NULL);
		double t0=start.tv_sec+start.tv_usec/1000000.0;
		double t1=stop.tv_sec+stop.tv_usec/1000000.0;
		double dt=t1-t0;
		fprintf(stderr,"%.3f seconds for %d iter (%f sec/iter)\n",
			dt,niter,dt/niter);
	}
	
	return(0);
}
