/*
 * areatree.cc
 * 
 * Area tree density & mass input dialog. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "areatree.h"

#include <stdio.h>
#include <assert.h>

#include <Qt/qlayout.h>
#include <Qt/qspinbox.h>
#include <Qt/qlabel.h>
#include <Qt/qtooltip.h>
#include <Qt/qvalidator.h>
#include <Qt/qlineedit.h>


void AreaTreeDialog::_UpdateValues()
{
	SimSetupPars ssp;
	ssp.valid=false;
	
	emit sim_setup_pars_request(&ssp);
	assert(ssp.valid);
	
	if(tree_prop)
	{
		char tmp[32];
		snprintf(tmp,31,"%6f",ssp.tree_prop);
		char *end=tmp+strlen(tmp)-1;
		while(end>tmp && *end=='0')
		{  *(end--)='\0';  }
		if(*end=='.')
		{  *(++end)='0';  *(++end)='\0';  }
		tree_prop->setText(tmp);
		//tree_prop->setCursorPosition(2);
	}
	
	tree_mass->setValue(ssp.tree_mass);
}


// Assumes that ssp is valid; only updates values. 
void AreaTreeDialog::ReadValues(SimSetupPars *ssp)
{
	emit sim_setup_pars_request(ssp);
	assert(ssp->valid);
	
	if(tree_prop)
	{
		bool ok;
		double val=tree_prop->text().toDouble(&ok);
		if(ok && !(val<0.0 || val>1.0))
		{  ssp->tree_prop=val;  }
	}
	
	ssp->tree_mass=tree_mass->value();
}

bool AreaTreeDialog::YouHaveFocus()
{
	return(tree_mass->hasFocus() || (tree_prop && tree_prop->hasFocus()));
}

void AreaTreeDialog::YouGetFocus(int which)
{
	if(which)
	{  tree_mass->setFocus();  }
	else if(tree_prop)
	{  tree_prop->setFocus();  }
}


AreaTreeDialog::AreaTreeDialog(int with_prop,QWidget *_parent,const char *name,WFlags f) : 
	QWidget(_parent,name,f)
{
	QWidget *parent=this;
	
	if(with_prop)
	{
		tree_prop=new QLineEdit(parent);
		tree_prop->setValidator(new QDoubleValidator(0.0,1.0,6,tree_prop));
	}
	else
	{  tree_prop=NULL;  }
	
	tree_mass=new QSpinBox(1,10000,1,parent);
	  connect(tree_mass,SIGNAL(valueChanged(int)),
	    this,SIGNAL(sim_setup_pars_change()));
	
	QWidget *wtmp;
	QString _tdens("Tree density 0..1");
	QString _tmass("Number of time ticks a tree burns");
	QGridLayout *area_inplayout=new QGridLayout(this,3,2);
	  if(with_prop)
	  {
	    area_inplayout->addWidget(wtmp=new QLabel(tree_prop,"&Tree dens:",parent),0,0);
	    QToolTip::add(wtmp,_tdens);
	    area_inplayout->addWidget(tree_prop,0,1);
	    area_inplayout->addRowSpacing(1,5);
	  }
	  area_inplayout->addWidget(wtmp=new QLabel(tree_mass,"Tree &mass:",parent),2,0);
	  QToolTip::add(wtmp,_tmass);
	  area_inplayout->addWidget(tree_mass,2,1);
}

AreaTreeDialog::~AreaTreeDialog()
{
}
