/*
 * areatree.h
 * 
 * Area tree density & mass input dialog. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _BURN_GUI_AREATREE_H_
#define _BURN_GUI_AREATREE_H_ 1

//#include <burncore/burncore.h>

#include <Qt/qwidget.h>

class QSpinBox;
class QLineEdit;


struct SimSetupPars
{
	double tree_prop;
	int tree_mass;
	bool valid;
};

class AreaTreeDialog : public QWidget
{
	Q_OBJECT
	private:
		QLineEdit *tree_prop;
		QSpinBox *tree_mass;
		
		void _UpdateValues();
	private slots:
	public slots:
	signals:
		// Called so that the current pars are filled in. 
		// MUST BE CONNECTED. 
		void sim_setup_pars_request(SimSetupPars *);
		// Called on value change. Needed by stburn. 
		// Changing tree_prop will NOT emit this signal. 
		void sim_setup_pars_change();
	public:
		AreaTreeDialog(int with_prop,QWidget *parent,const char *name=NULL,WFlags f=0);
		~AreaTreeDialog();
		
		void Update()
			{  _UpdateValues();  }
		
		// Get current values from sim setup and read in values 
		// as set by user, then return them in ssp. 
		// You should call Update() after that. 
		void ReadValues(SimSetupPars *ssp);
		
		bool YouHaveFocus();
		void YouGetFocus(int which);
};

#endif  /* _BURN_GUI_AREATREE_H_ */
