/*
 * ignpars.cc
 * 
 * Tree ignite parameter widget. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "ignpars.h"

#include <burncore/burncore.h>

#include <Qt/qspinbox.h>
#include <Qt/qlineedit.h>
#include <Qt/qpushbutton.h>
#include <Qt/qcheckbox.h>
#include <Qt/qgroupbox.h>
#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qvalidator.h>
#include <Qt/qtooltip.h>

#include <stdlib.h>
#include <assert.h>


void IgniteParsDialog::DoAutoIgnite(BurnArea *area)
{
	if(auto_ignition && !auto_ignition->isOn())  return;
	
	if(v_change)
	{  _GetValues();  }
	
	int aw=area->AreaWidth();
	int ah=area->AreaHeight();
	
	int xpos=int( double(aw)*double(xpos_v)/100.0 + 0.5 );
	int ypos=int( double(ah)*double(ypos_v)/100.0 + 0.5 );
	     if(xpos<0)    xpos=0;
	else if(xpos>=aw)  xpos=aw-1;
	     if(ypos<0)    ypos=0;
	else if(ypos>=ah)  ypos=ah-1;
	
	// This is the number of cells to set fire on: 
	int cnt=size_v;
	
	// See if we can set fire on the cell at xpos,ypos: 
	if(!area->SetFire(xpos,ypos))  --cnt;
	
	// Do rect loops around this position: 
	int aend = ((aw<ah) ? ah : aw)-1;  // hard limit
	if(cnt) for(int a=1; a<aend; a++)  // a*2+1 = rect size
	{
		// Yeah... some people may say that this is ugly. 
		#define _TrySetFire \
			if(!area->SetFire(xx,yy)) \
			{	if(!(--cnt))  goto done;  } \
		
		if(area->AreaIsTorus())
		{
			for(int i=-a; i<=a; i++)
			{
				register int xx,yy;
				xx=(xpos+a   )%aw;  yy=(ypos+i+ah)%ah;  _TrySetFire
				xx=(xpos-a+aw)%aw;  yy=(ypos+i+ah)%ah;  _TrySetFire
				xx=(xpos+i+aw)%aw;  yy=(ypos+a   )%ah;  _TrySetFire
				xx=(xpos+i+aw)%aw;  yy=(ypos-a+ah)%ah;  _TrySetFire
			}
		}
		else
		{
			for(int i=-a; i<=a; i++)
			{
				register int xx,yy;
				xx=xpos+a;  yy=ypos+i;  _TrySetFire
				xx=xpos-a;  yy=ypos+i;  _TrySetFire
				xx=xpos+i;  yy=ypos+a;  _TrySetFire
				xx=xpos+i;  yy=ypos-a;  _TrySetFire
			}
		}
		
		#undef _TrySetFire
	}
	done:;
}


void IgniteParsDialog::_GetValues()
{
	xpos_v=xposi->value();
	ypos_v=yposi->value();
	size_v=sizei->value();
	assert(size_v);
	v_change=0;
}


void IgniteParsDialog::v_change_slot()
{
	v_change=1;
}


void IgniteParsDialog::map()
{
	if(!isVisible())
	{  show();  return;  }
	
	if(!auto_ignition || auto_ignition->isOn())
	{  emit req_auto_ignite_now();  }
	
	hide();
}


IgniteParsDialog::IgniteParsDialog(QWidget *_parent,const char *name,WFlags f) : 
	QWidget(_parent,name,f)
{
	QWidget *parent=this;
	
	QPushButton *close_button=NULL;
	QHBoxLayout *close_layout=NULL;
	
	if(_parent)
	{
		parent=new QGroupBox("Ignition",this);
		
		QVBoxLayout *toplayout=new QVBoxLayout(this);
		toplayout->addSpacing(5);
		toplayout->addWidget(parent);
		
		auto_ignition=NULL;
	}
	else
	{  auto_ignition=new QCheckBox("&Auto ignition",parent);  }
	
	static const char *_pof_str_x="X position of fire as percentage\nof total width.";
	static const char *_pof_str_y="Y position of fire as percentage\nof total height.";
	static const char *_pof_str_s="Initial size of the fire (number\nof burning trees).";
	xposi=new QSpinBox(0,100,1,parent);
	  xposi->setSuffix("%");
	  connect(xposi,SIGNAL(valueChanged(int)),this,SLOT(v_change_slot()));
	  QToolTip::add(xposi,_pof_str_x);
	yposi=new QSpinBox(0,100,1,parent);
	  yposi->setSuffix("%");
	  connect(yposi,SIGNAL(valueChanged(int)),this,SLOT(v_change_slot()));
	  QToolTip::add(yposi,_pof_str_y);
	sizei=new QSpinBox(1,100,1,parent);
	  connect(sizei,SIGNAL(valueChanged(int)),this,SLOT(v_change_slot()));
	  QToolTip::add(sizei,_pof_str_s);
	
	QLabel *xposl=new QLabel(xposi,"&X pos:",parent);
	  QToolTip::add(xposl,_pof_str_x);
	QLabel *yposl=new QLabel(yposi,"&Y pos:",parent);
	  QToolTip::add(yposl,_pof_str_y);
	QLabel *sizel=new QLabel(yposi,_parent ? "&Initial size:" : "&Size:",parent);
	  QToolTip::add(sizel,_pof_str_s);
	
	QGridLayout *inplayout=new QGridLayout(3,2);
	  inplayout->addWidget(xposl,0,0);
	  inplayout->addWidget(yposl,1,0);
	  inplayout->addWidget(sizel,2,0);
	  inplayout->addWidget(xposi,0,1);
	  inplayout->addWidget(yposi,1,1);
	  inplayout->addWidget(sizei,2,1);
	
	if(!_parent)
	{
		QToolTip::add(auto_ignition,"Enable/disable auto ignition.");
		connect(auto_ignition,SIGNAL(toggled(bool)),xposi,SLOT(setEnabled(bool)));
		connect(auto_ignition,SIGNAL(toggled(bool)),yposi,SLOT(setEnabled(bool)));
		connect(auto_ignition,SIGNAL(toggled(bool)),sizei,SLOT(setEnabled(bool)));
		connect(auto_ignition,SIGNAL(toggled(bool)),xposl,SLOT(setEnabled(bool)));
		connect(auto_ignition,SIGNAL(toggled(bool)),yposl,SLOT(setEnabled(bool)));
		connect(auto_ignition,SIGNAL(toggled(bool)),sizel,SLOT(setEnabled(bool)));
	}
	
	QVBoxLayout *toplayout=new QVBoxLayout(parent,4,4);
	if(auto_ignition)
	{  toplayout->addWidget(auto_ignition);  }
	if(_parent)
	{  toplayout->addSpacing(15);  }
	toplayout->addLayout(inplayout);
	if(!_parent)
	{
		close_button=new QPushButton("&Close",parent);
		  connect(close_button,SIGNAL(clicked()),this,SLOT(map()));
		
		close_layout=new QHBoxLayout();
		  close_layout->addWidget(close_button);
		
		toplayout->addSpacing(20);
		toplayout->addLayout(close_layout);
	}
	
	if(auto_ignition)
	{
		auto_ignition->setDown(false);
		xposi->setEnabled(false);
		yposi->setEnabled(false);
		sizei->setEnabled(false);
		xposl->setEnabled(false);
		yposl->setEnabled(false);
		sizel->setEnabled(false);
	}
	
	xposi->setValue(50);
	yposi->setValue(50);
	sizei->setValue(5);
	
	v_change=1;
}


IgniteParsDialog::~IgniteParsDialog()
{
	
}

