/*
 * areawindow.h
 * 
 * Qxburn's simulation area window. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _BURN_QTXGUI_AREAWINDOW_H_
#define _BURN_QTXGUI_AREAWINDOW_H_ 1

#include <burncore/burncore.h>
#include "../gui/areasetup.h"
#include "statwindow.h"

#include <Qt/qwidget.h>

#include <QTX/bufrectpainterarray.h>
#include <QTX/bufpointpainterarray.h>

#include "../gui/areatree.h"


class MainWindow;
struct AreaSetupPars;
class IgniteParsDialog;

// Callback for BurnArea: 
extern void _UpdateHandler(void *thisptr,int x,int y,BCell *c);


class AreaWindow : public QWidget
{
	Q_OBJECT
	private:
		int running;
		BurnArea area;
		SimSetupPars ssp;
		
		int cellx0,celly0;
		uint cellw,cellh;
		
		bool use_ppnt;
		QTX::BufferedRectPainterArray rpnt;
		QTX::BufferedPointPainterArray ppnt;
		
		int delay_mode;  // 0 -> msec delay per tick; 1 -> ticks per delay
		                 // ALWAYS 0 if delay_val is 0. 
		int delay_val;
		int ticks_since_delay;
		
		// Iterations left to do. -1 for unlimited. 
		int iterations_left;
		
		// If there are no more trees. 
		int no_more_trees;
		
		// Events (overriding virtuals): 
		void timerEvent(QTimerEvent *);
		void paintEvent(QPaintEvent *);
		void resizeEvent(QResizeEvent *);
		void mouseMoveEvent(QMouseEvent *);
		void mousePressEvent(QMouseEvent *);
		void keyPressEvent(QKeyEvent *);
		void keyReleaseEvent(QKeyEvent *);
		
		#if USE_STATE_CHG
		void _UpdateChangedCells();
		#else
		friend void _UpdateHandler(void *thisptr,int x,int y,BCell *c);
		inline void _UpdateCell(int x,int y,BCell *c);
		#endif
		
		StatusWindow *statwin;
		size_t tick_cnt;
		inline void _UpdateStat()
			{  if(statwin) statwin->Update(tick_cnt,&area);  }
	signals:
		void sim_has_stopped(int);
		void key_press_event(QKeyEvent *);
		void key_release_event(QKeyEvent *);
	public slots:
		// called to query area setup/sim params: 
		void fillin_area_setup(AreaSetupPars *spar);
		void fillin_sim_setup(SimSetupPars *spar);
		void fillin_burnprop(double *dest);
		// called when area setup changes: 
		void area_setup_changed(AreaSetupPars *spar);
	public:
		AreaWindow(QWidget *_parent=NULL,
			const char *name=NULL,WFlags f=0);
		~AreaWindow();
		
		void SetStatWindow(StatusWindow *swin)
			{  statwin=swin;  }
		
		int Running()
			{  return(running);  }
		
		// Show/hide window: 
		void Map();
		
		// Sim control: 
		void ContSim(int iteration_limit);
		void StopSim();
		
		void SetDelay(int delay_mode,int delay_val);
		void ChangeBurnProp(const double *prop);
		
		void ClearArea(SimSetupPars *ssp);
		void DoAutoIgnite(IgniteParsDialog *ignpars);
		
		// Re-draw the screen: 
		void Update();
		
		// Returns 1, if at least one tree is burning. 
		int HasBurningTrees()
			{  return(area.HasBurningCells());  }
		// The same for dead trees: 
		int HasDeadTrees()
			{  return(area.HasDeadCells());  }
};

#endif  /* _BURN_QTXGUI_AREAWINDOW_H_ */
