/*
 * mainwindow.h
 * 
 * Qxburn's main window. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _BURN_QTXGUI_MAINWINDOW_H_
#define _BURN_QTXGUI_MAINWINDOW_H_ 1

#include "areawindow.h"
#include "../gui/ignpars.h"
#include "../gui/burnprop.h"
#include "../gui/areatree.h"

#include <Qt/qwidget.h>


class QPushButton;
class QComboBox;

class StatusWindow;


class MainWindow : public QWidget
{
	Q_OBJECT
	private:
		AreaWindow areawin;
		AreaSetupDialog asetup;
		IgniteParsDialog ignitepars;
		BurnPropDialog *bpropd;
		AreaTreeDialog *atd;
		
		QPushButton *area_button,*ignite_button;
		QPushButton *reset_button;
		QPushButton *stop_cont_button;
		
		QComboBox *delay_mode;
		QSpinBox *delay_time;
		
		QSpinBox *n_iterations;
		
		StatusWindow *swin;
		
		void _UpdateValues();
		
		void _HandleKeyEvent(QKeyEvent *ev,int stat);
		
		void timerEvent(QTimerEvent *);
		void keyPressEvent(QKeyEvent *);
		void keyReleaseEvent(QKeyEvent *);
	private slots:
		void key_press_slot(QKeyEvent *);
		void key_release_slot(QKeyEvent *);
		void quit_clicked();
		void info_clicked();
		void clear_clicked();
		void area_setup_changed(AreaSetupPars*);
		void delay_change();
		// start/stop/cont button clicked: 
		void stop_cont_clicked();
		void sim_has_stopped(int);
		void do_auto_ignite_req_now();
		void burn_prop_change(double *bprop);
		void burn_prop_request_slot(double *bprop);
		void sim_setup_pars_request_slot(SimSetupPars *);
	public:
		MainWindow(QWidget *_parent=NULL,const char *name=NULL,WFlags f=0);
		~MainWindow();
};

#endif  /* _BURN_QTXGUI_MAINWINDOW_H_ */
