/*
 * statwindow.cc
 * 
 * VERY simple statistics widget. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "statwindow.h"

#include <stdio.h>


void StatusWindow::_Redraw()
{
	//DrawString(x,y,str,len);
	
	int yy=13,yinc=14;
	
	DrawString(5,yy+=yinc,"Time tick:");
	DrawString(5,yy+=yinc+3,"      Living    Burning   Dead");
	DrawString(5,yy+=yinc,  "Abs:");
	DrawString(5,yy+=yinc,  "Rel:");
	
	_Update();
}


void StatusWindow::_Update()
{
	int yy=13,yinc=14;
	
	char tmp[64];
	snprintf(tmp,64,"%u",tick);
	backpnt.FillRectangle(5+7*10-1,(yy+=yinc)-10,80,10);
	DrawString(5+7*10,yy,tmp);
	yy+=yinc+3;
	
	backpnt.FillRectangle(5+4*7-1,(yy+=yinc)-10,175,10+yinc);
	snprintf(tmp,64,"%8u  %8u  %8u",living,burning,dead);
	DrawString(5+5*7,yy,tmp);
	
	size_t stot=living+burning+dead;  // WITHOUT empty
	if(stot)
	{
		double tot=double(stot);
		int pliv= int(double(living)/tot*100000.0+0.5);
		int pburn=int(double(burning)/tot*100000.0+0.5);
		int pdead=int(double(dead)/tot*100000.0+0.5);
		snprintf(tmp,64,"%3d.%03d%%  %3d.%03d%%  %3d.%03d%%",
			pliv/1000,pliv%1000,
			pburn/1000,pburn%1000,
			pdead/1000,pdead%1000);
		DrawString(5+5*7,yy+=yinc,tmp);
	}
}


void StatusWindow::Update(size_t _tick,BurnArea *area)
{
	tick=_tick;
	area->GetNCells(&empty,&living,&burning,&dead);
	_Update();
}


void StatusWindow::paintEvent(QPaintEvent *ev)
{
	QGroupBox::paintEvent(ev);
	
	_Redraw();
}


StatusWindow::StatusWindow(QWidget *parent) : 
	QGroupBox("Status",parent),
	QTX::XPainter(this),
	backpnt(this)
{
	fnt=XLoadFont(dpy,"fixed");
	SetFont(fnt);
	
	backpnt.SetForeground(paletteBackgroundColor().pixel());
	
	tick=0;
	empty=living=burning=dead=0;
	
	int minw=214,minh=80;
	setMinimumSize(QSize(minw,minh));
}


StatusWindow::~StatusWindow()
{
	XUnloadFont(dpy,fnt);
}
