/*
 * mainwindow.h
 * 
 * Stburn's main window. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _BURN_STBURN_MAINWINDOW_H_
#define _BURN_STBURN_MAINWINDOW_H_ 1

#include <burncore/burncore.h>

#include "../gui/areasetup.h"
#include "../gui/areatree.h"

#include <Qt/qwidget.h>

#include <time.h>
#include <sys/time.h>


class QPushButton;
class QComboBox;

class IgniteParsDialog;
class AreaTreeDialog;
class BurnPropDialog;
class StatisticsDiagram;

struct AreaSetupPars;


class MainWindow : public QWidget
{
	Q_OBJECT
	private:
		BurnArea area;
		int iterations;
		SimSetupPars ssp;  // of currently running sim 
		int running;
		
		AreaSetupDialog asetup;
		IgniteParsDialog *ignitepars;
		AreaTreeDialog *atd;
		BurnPropDialog *bpropd;
		QSpinBox *diag_limit;
		
		QPushButton *area_button;
		QPushButton *stop_cont_button;
		
		StatisticsDiagram *stat_diag;
		
		// For sps (sims per second): 
		int sps_time_valid;
		timeval sps_starttime;
		
		RandomGenerator rndgen;
		int sweep_step;
		int sweep_iter;
		double sweep2_center,sweep2_delta;
		double _GetTreeProp();
		
		void _UpdateValues();
		
		void _StopSim();
		void _ContSim();
		
		void _HandleKeyEvent(QKeyEvent *ev,int stat);
		
		void timerEvent(QTimerEvent *);
		void keyPressEvent(QKeyEvent *);
		void keyReleaseEvent(QKeyEvent *);
	private slots:
		void quit_clicked();
		void info_clicked();
		void reset_clicked();
		void area_setup_changed(AreaSetupPars*);
		// start/stop/cont button clicked: 
		void stop_cont_clicked();
		void burn_prop_change(double *bprop);
		void burn_prop_request_slot(double *bprop);
		void sim_setup_pars_request_slot(SimSetupPars *);
		void sim_setup_pars_changed();
		void fillin_area_setup(AreaSetupPars*);
		void diag_limit_changed(int);
	public:
		MainWindow(QWidget *_parent=NULL,const char *name=NULL,WFlags f=0);
		~MainWindow();
};

#endif  /* _BURN_STBURN_MAINWINDOW_H_ */
